@@ -3,7 +3,8 @@
 #                                                                         #
 # The purpose of the caterpillar.plot function is to provide a            #
 # caterpillar plot of the posterior summaries in an object of class       #
-# demonoid or laplace.                                                    #
+# demonoid or laplace, or also of S x J matrix of S samples and J         #
+# variables.                                                              #
 ###########################################################################
 
 caterpillar.plot <- function(x, Parms=NULL, Title=NULL)
@@ -116,7 +117,35 @@ caterpillar.plot <- function(x, Parms=NULL, Title=NULL)
           axis(2, labels=rownames(x), tick=FALSE, las=1,
                at=yy, cex.axis=cex.labels)
           }
-     else stop("x must be of class demonoid or laplace.")
+     else {
+          x <- as.matrix(x)
+          x.hpd <- p.interval(x, HPD=TRUE, MM=FALSE, prob=0.95)
+          x.median <- apply(x, 2, median)
+          x <- cbind(apply(x, 2, mean), apply(x, 2, sd),
+               apply(x, 2, MCSE), ESS(x), x.hpd[,1],
+               apply(x, 2, median), x.hpd[,2])
+          rownames(x) <- rownames(x.hpd)
+          colnames(x) <- c("Mean","SD","MCSE","ESS","LB","Median","UB")
+          x.lab <- "All Samples"
+          ### Setup
+          x.rows <- nrow(x)
+          x.lim <- c(min(x[,5]), max(x[,7]))
+          y.lim <- c(0, x.rows+1)
+          ### Basic Plot
+          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
+               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
+          abline(v=0, col="gray")
+          ### Add Medians
+          points(x[,6], x.rows:1, pch=20)
+          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
+          for (i in 1:x.rows) {
+               lines(x[i,c(5,7)], c(x.rows-i+1, x.rows-i+1))}
+          ### Add y-axis labels
+          yy <- x.rows:1
+          cex.labels <- 1 / {log(x.rows)/5 + 1}
+          axis(2, labels=rownames(x), tick=FALSE, las=1, at=yy,
+               cex.axis=cex.labels)
+          }
      }
 
 #End
