@@ -10,9 +10,11 @@ plot.laplace <- function(x, Data=NULL, PDF=FALSE, Parms=NULL, ...)
      ### Initial Checks
      if(missing(x)) stop("The x argument is required.")
      if(is.null(Data)) stop("The Data argument is NULL.")
-     if(is.na(x$History)) stop("There is no history to plot.")
+     if(any(is.na(x$History))) stop("There is no history to plot.")
      ### Selecting Parms
-     if(is.null(Parms)) History <- x$History
+     if(is.null(Parms)) {
+          History <- x$History
+          Posterior <- x$Posterior}
      else {
           Parms <- sub("\\[","\\\\[",Parms)
           Parms <- sub("\\]","\\\\]",Parms)
@@ -29,23 +31,40 @@ plot.laplace <- function(x, Data=NULL, PDF=FALSE, Parms=NULL, ...)
                          colnames(x$History)))}}
           History <- as.matrix(x$History[,keepcols])
           colnames(History) <- colnames(x$History)[keepcols]
+          Posterior <- as.matrix(x$Posterior[,keepcols])
+          colnames(Posterior) <- colnames(History)
           }
      if(PDF == TRUE)
           {
           pdf("LaplaceApproximation.Plots.pdf")
-          par(mfrow=c(3,3))
+          par(mfrow=c(2,2))
           }
-     else {par(mfrow=c(3,3), ask=TRUE)}
-     ### Plot Parameters
-     for (j in 1:NCOL(History))
+     else {par(mfrow=c(2,2), ask=TRUE)}
+     ### Plot Parameter
+     for (j in 1:ncol(History))
           {
-          plot(1:NROW(History), History[,j],
+          plot(1:nrow(History), History[,j],
                type="l", xlab="Iterations", ylab="Value",
                main=colnames(History)[j])
+          if({x$Converged == TRUE} & !any(is.na(x$Posterior))) {
+               plot(density(Posterior[,j]),
+                    xlab="Value", main=colnames(Posterior)[j])
+               polygon(density(Posterior[,j]),
+                    col="black", border="black")
+               abline(v=0, col="red", lty=2)}
           }
-     ### Plot Deviance
+     ### Plot Deviance History
      plot(1:length(x$Deviance), x$Deviance, type="l", xlab="Iterations",
           ylab="Value", main="Deviance")
+     ### Plot Monitor
+     if({x$Converged == TRUE} & !any(is.na(x$Monitor))) {
+          for (j in 1:ncol(x$Monitor)) {
+               plot(density(x$Monitor[,j]),
+                    xlab="Value", main=Data$mon.names[j])
+               polygon(density(x$Monitor[,j]),
+                    col="black", border="black")
+               abline(v=0, col="red", lty=2)}
+          }
      if(PDF == TRUE) dev.off()
      }
 
